@@ -67,6 +67,7 @@ from ansiblelint.config import options
 from ansiblelint.constants import NESTED_TASK_KEYS, PLAYBOOK_TASK_KEYWORDS, FileType
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, discover_lintables
+from ansiblelint.skip_utils import is_nested_task
 from ansiblelint.text import removeprefix
 
 # ansible-lint doesn't need/want to know about encrypted secrets, so we pass a
@@ -463,7 +464,7 @@ def _rolepath(basedir: str, role: str) -> Optional[str]:
         path_dwim(basedir, os.path.join("..", role)),
     ]
 
-    for loc in get_app().runtime.config.default_roles_path:
+    for loc in get_app(offline=True).runtime.config.default_roles_path:
         loc = os.path.expanduser(loc)
         possible_paths.append(path_dwim(loc, role))
 
@@ -897,12 +898,3 @@ def nested_items(
             yield "list-item", item, parent
             for k, v, returned_parent in nested_items(item):
                 yield k, v, returned_parent
-
-
-def is_nested_task(task: Dict[str, Any]) -> bool:
-    """Check if task includes block/always/rescue."""
-    for key in NESTED_TASK_KEYS:
-        if task.get(key):
-            return True
-
-    return False
