@@ -1,10 +1,12 @@
 """Utility helpers to simplify working with yaml-based data."""
+# pylint: disable=too-many-lines
 import functools
 import logging
 import os
 import re
 from io import StringIO
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
@@ -12,12 +14,14 @@ from typing import (
     List,
     Optional,
     Pattern,
+    Set,
     Tuple,
     Union,
     cast,
 )
 
 import ruamel.yaml.events
+from ruamel.yaml.comments import CommentedMap, CommentedSeq, Format
 from ruamel.yaml.constructor import RoundTripConstructor
 from ruamel.yaml.emitter import Emitter, ScalarAnalysis
 
@@ -31,10 +35,15 @@ from ruamel.yaml.tokens import CommentToken
 from yamllint.config import YamlLintConfig
 
 import ansiblelint.skip_utils
+from ansiblelint.constants import NESTED_TASK_KEYS, PLAYBOOK_TASK_KEYWORDS
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable
 from ansiblelint.utils import get_action_tasks, normalize_task, parse_yaml_linenumbers
 
+if TYPE_CHECKING:
+    # noinspection PyProtectedMember
+    from ruamel.yaml.comments import LineCol  # pylint: disable=ungrouped-imports
+
 _logger = logging.getLogger(__name__)
 
 YAMLLINT_CONFIG = """
@@ -50,6 +59,11 @@ rules:
   # you can easily change it or disable in your .yamllint file.
   line-length:
     max: 160
+  # We are adding an extra space inside braces as that's how prettier does it
+  # and we are trying not to fight other linters.
+  braces:
+    min-spaces-inside: 0  # yamllint defaults to 0
+    max-spaces-inside: 1  # yamllint defaults to 0
 """
 
 
@@ -59,9 +73,16 @@ def load_yamllint_config() -> YamlLintConfig:
     config = YamlLintConfig(content=YAMLLINT_CONFIG)
     # if we detect local yamllint config we use it but raise a warning
     # as this is likely to get out of sync with our internal config.
-    for file in [".yamllint", ".yamllint.yaml", ".yamllint.yml"]:
+    for file in [
+        ".yamllint",
+        ".yamllint.yaml",
+        ".yamllint.yml",
+        os.getenv("YAMLLINT_CONFIG_FILE", ""),
+        os.getenv("XDG_CONFIG_HOME", os.path.expanduser("~/.config"))
+        + "/yamllint/config",
+    ]:
         if os.path.isfile(file):
-            _logger.warning(
+            _logger.debug(
                 "Loading custom %s config file, this extends our "
                 "internal yamllint config.",
                 file,
@@ -225,6 +246,215 @@ def _nested_items_path(
             )
 
 
+def get_path_to_play(
+    lintable: Lintable,
+    line_number: int,  # 1-based
+    ruamel_data: Union[CommentedMap, CommentedSeq],
+) -> List[Union[str, int]]:
+    """Get the path to the play in the given file at the given line number."""
+    if line_number < 1:
+        raise ValueError(f"expected line_number >= 1, got {line_number}")
+    if lintable.kind != "playbook" or not isinstance(ruamel_data, CommentedSeq):
+        return []
+    lc: "LineCol"  # lc uses 0-based counts # pylint: disable=invalid-name
+    # line_number is 1-based. Convert to 0-based.
+    line_index = line_number - 1
+
+    prev_play_line_index = ruamel_data.lc.line
+    last_play_index = len(ruamel_data)
+    for play_index, play in enumerate(ruamel_data):
+        next_play_index = play_index + 1
+        if last_play_index > next_play_index:
+            next_play_line_index = ruamel_data[next_play_index].lc.line
+        else:
+            next_play_line_index = None
+
+        lc = play.lc  # pylint: disable=invalid-name
+        assert isinstance(lc.line, int)
+        if lc.line == line_index:
+            return [play_index]
+        if play_index > 0 and prev_play_line_index < line_index < lc.line:
+            return [play_index - 1]
+        # The previous play check (above) can't catch the last play,
+        # so, handle the last play separately.
+        if (
+            next_play_index == last_play_index
+            and line_index > lc.line
+            and (next_play_line_index is None or line_index < next_play_line_index)
+        ):
+            # part of this (last) play
+            return [play_index]
+        prev_play_line_index = play.lc.line
+    return []
+
+
+def get_path_to_task(
+    lintable: Lintable,
+    line_number: int,  # 1-based
+    ruamel_data: Union[CommentedMap, CommentedSeq],
+) -> List[Union[str, int]]:
+    """Get the path to the task in the given file at the given line number."""
+    if line_number < 1:
+        raise ValueError(f"expected line_number >= 1, got {line_number}")
+    if lintable.kind in ("tasks", "handlers"):
+        assert isinstance(ruamel_data, CommentedSeq)
+        return _get_path_to_task_in_tasks_block(line_number, ruamel_data)
+    if lintable.kind == "playbook":
+        assert isinstance(ruamel_data, CommentedSeq)
+        return _get_path_to_task_in_playbook(line_number, ruamel_data)
+    # if lintable.kind in ["yaml", "requirements", "vars", "meta", "reno", "test-meta"]:
+
+    return []
+
+
+def _get_path_to_task_in_playbook(
+    line_number: int,  # 1-based
+    ruamel_data: CommentedSeq,
+) -> List[Union[str, int]]:
+    """Get the path to the task in the given playbook data at the given line number."""
+    last_play_index = len(ruamel_data)
+    for play_index, play in enumerate(ruamel_data):
+        next_play_index = play_index + 1
+        if last_play_index > next_play_index:
+            next_play_line_index = ruamel_data[next_play_index].lc.line
+        else:
+            next_play_line_index = None
+
+        play_keys = list(play.keys())
+        for tasks_keyword in PLAYBOOK_TASK_KEYWORDS:
+            if not play.get(tasks_keyword):
+                continue
+
+            try:
+                next_keyword = play_keys[play_keys.index(tasks_keyword) + 1]
+            except IndexError:
+                next_block_line_index = None
+            else:
+                next_block_line_index = play.lc.data[next_keyword][0]
+            # last_line_number_in_block is 1-based; next_*_line_index is 0-based
+            # next_*_line_index - 1 to get line before next_*_line_index.
+            # Then + 1 to make it a 1-based number.
+            # So, last_line_number_in_block = next_*_line_index - 1 + 1
+            if next_block_line_index is not None:
+                last_line_number_in_block = next_block_line_index
+            elif next_play_line_index is not None:
+                last_line_number_in_block = next_play_line_index
+            else:
+                last_line_number_in_block = None
+
+            task_path = _get_path_to_task_in_tasks_block(
+                line_number, play[tasks_keyword], last_line_number_in_block
+            )
+            if task_path:
+                # mypy gets confused without this typehint
+                tasks_keyword_path: List[Union[int, str]] = [
+                    play_index,
+                    tasks_keyword,
+                ]
+                return tasks_keyword_path + list(task_path)
+    # line_number is before first play or no tasks keywords in any of the plays
+    return []
+
+
+def _get_path_to_task_in_tasks_block(
+    line_number: int,  # 1-based
+    tasks_block: CommentedSeq,
+    last_line_number: Optional[int] = None,  # 1-based
+) -> List[Union[str, int]]:
+    """Get the path to the task in the given tasks block at the given line number."""
+    task: Optional[CommentedMap]
+    # line_number and last_line_number are 1-based. Convert to 0-based.
+    line_index = line_number - 1
+    last_line_index = None if last_line_number is None else last_line_number - 1
+
+    # lc (LineCol) uses 0-based counts
+    prev_task_line_index = tasks_block.lc.line
+    last_task_index = len(tasks_block)
+    for task_index, task in enumerate(tasks_block):
+        next_task_index = task_index + 1
+        if last_task_index > next_task_index:
+            if tasks_block[next_task_index] is not None:
+                next_task_line_index = tasks_block[next_task_index].lc.line
+            else:
+                next_task_line_index = tasks_block.lc.item(next_task_index)[0]
+        else:
+            next_task_line_index = None
+
+        if task is None:
+            # create a dummy task to represent the null task
+            task = CommentedMap()
+            task.lc.line, task.lc.col = tasks_block.lc.item(task_index)
+
+        nested_task_keys = set(task.keys()).intersection(set(NESTED_TASK_KEYS))
+        if nested_task_keys:
+            subtask_path = _get_path_to_task_in_nested_tasks_block(
+                line_number, task, nested_task_keys, next_task_line_index
+            )
+            if subtask_path:
+                # mypy gets confused without this typehint
+                task_path: List[Union[str, int]] = [task_index]
+                return task_path + list(subtask_path)
+
+        assert isinstance(task.lc.line, int)
+        if task.lc.line == line_index:
+            return [task_index]
+        if task_index > 0 and prev_task_line_index < line_index < task.lc.line:
+            return [task_index - 1]
+        # The previous task check can't catch the last task,
+        # so, handle the last task separately (also after subtask checks).
+        # pylint: disable=too-many-boolean-expressions
+        if (
+            next_task_index == last_task_index
+            and line_index > task.lc.line
+            and (next_task_line_index is None or line_index < next_task_line_index)
+            and (last_line_index is None or line_index <= last_line_index)
+        ):
+            # part of this (last) task
+            return [task_index]
+        prev_task_line_index = task.lc.line
+    # line is not part of this tasks block
+    return []
+
+
+def _get_path_to_task_in_nested_tasks_block(
+    line_number: int,  # 1-based
+    task: CommentedMap,
+    nested_task_keys: Set[str],
+    next_task_line_index: Optional[int] = None,  # 0-based
+) -> List[Union[str, int]]:
+    """Get the path to the task in the given nested tasks block."""
+    # loop through the keys in line order
+    task_keys = list(task.keys())
+    task_keys_by_index = dict(enumerate(task_keys))
+    for task_index, task_key in enumerate(task_keys):
+        nested_task_block = task[task_key]
+        if task_key not in nested_task_keys or not nested_task_block:
+            continue
+        next_task_key = task_keys_by_index.get(task_index + 1, None)
+        if next_task_key is not None:
+            next_task_key_line_index = task.lc.data[next_task_key][0]
+        else:
+            next_task_key_line_index = None
+        # last_line_number_in_block is 1-based; next_*_line_index is 0-based
+        # next_*_line_index - 1 to get line before next_*_line_index.
+        # Then + 1 to make it a 1-based number.
+        # So, last_line_number_in_block = next_*_line_index - 1 + 1
+        last_line_number_in_block = (
+            next_task_key_line_index
+            if next_task_key_line_index is not None
+            else next_task_line_index
+        )
+        subtask_path = _get_path_to_task_in_tasks_block(
+            line_number,
+            nested_task_block,
+            last_line_number_in_block,  # 1-based
+        )
+        if subtask_path:
+            return [task_key] + list(subtask_path)
+    # line is not part of this nested tasks block
+    return []
+
+
 class OctalIntYAML11(ScalarInt):
     """OctalInt representation for YAML 1.1."""
 
@@ -265,6 +495,9 @@ class CustomConstructor(RoundTripConstructor):
         """
         ret = super().construct_yaml_int(node)
         if self.resolver.processing_version == (1, 1) and isinstance(ret, int):
+            # Do not rewrite zero as octal.
+            if ret == 0:
+                return ret
             # see if we've got an octal we need to preserve.
             value_su = self.construct_scalar(node)
             try:
@@ -307,6 +540,9 @@ class FormattedEmitter(Emitter):
 
     preferred_quote = '"'  # either " or '
 
+    min_spaces_inside = 0
+    max_spaces_inside = 1
+
     _sequence_indent = 2
     _sequence_dash_offset = 0  # Should be _sequence_indent - 2
     _root_is_sequence = False
@@ -367,13 +603,20 @@ class FormattedEmitter(Emitter):
         indention: bool = False,  # (sic) ruamel.yaml has this typo in their API
     ) -> None:
         """Make sure that flow maps get whitespace by the curly braces."""
+        # We try to go with one whitespace by the curly braces and adjust accordingly
+        # to what min_spaces_inside and max_spaces_inside are set to.
+        # This assumes min_spaces_inside <= max_spaces_inside
+        spaces_inside = min(
+            max(1, self.min_spaces_inside),
+            self.max_spaces_inside if self.max_spaces_inside != -1 else 1,
+        )
         # If this is the end of the flow mapping that isn't on a new line:
         if (
             indicator == "}"
             and (self.column or 0) > (self.indent or 0)
             and not self._in_empty_flow_map
         ):
-            indicator = " }"
+            indicator = (" " * spaces_inside) + "}"
         super().write_indicator(indicator, need_whitespace, whitespace, indention)
         # if it is the start of a flow mapping, and it's not time
         # to wrap the lines, insert a space.
@@ -382,7 +625,7 @@ class FormattedEmitter(Emitter):
                 self._in_empty_flow_map = True
             else:
                 self.column += 1
-                self.stream.write(" ")
+                self.stream.write(" " * spaces_inside)
                 self._in_empty_flow_map = False
 
     # "/n/n" results in one blank line (end the previous line, then newline).
@@ -561,6 +804,9 @@ class FormattedYAML(YAML):
         indent_sequences: bool = cast(bool, config["indent_sequences"])
         preferred_quote: str = cast(str, config["preferred_quote"])  # either ' or "
 
+        min_spaces_inside: int = cast(int, config["min_spaces_inside"])
+        max_spaces_inside: int = cast(int, config["max_spaces_inside"])
+
         self.default_flow_style = False
         self.compact_seq_seq = True  # type: ignore[assignment] # dash after dash
         self.compact_seq_map = True  # type: ignore[assignment] # key after dash
@@ -579,6 +825,10 @@ class FormattedYAML(YAML):
         # NB: default_style affects preferred_quote as well.
         # self.default_style ∈ None (default), '', '"', "'", '|', '>'
 
+        # spaces inside braces for flow mappings
+        FormattedEmitter.min_spaces_inside = min_spaces_inside
+        FormattedEmitter.max_spaces_inside = max_spaces_inside
+
         # We need a custom constructor to preserve Octal formatting in YAML 1.1
         self.Constructor = CustomConstructor
         self.Representer.add_representer(OctalIntYAML11, OctalIntYAML11.represent_octal)
@@ -608,18 +858,28 @@ class FormattedYAML(YAML):
             "width": 160,
             "indent_sequences": True,
             "preferred_quote": '"',
+            "min_spaces_inside": 0,
+            "max_spaces_inside": 1,
         }
         for rule, rule_config in load_yamllint_config().rules.items():
             if not rule_config:
                 # rule disabled
                 continue
 
+            # refactor this if ... elif ... elif ... else monstrosity using match/case (PEP 634) once python 3.10 is mandatory
             if rule == "document-start":
                 config["explicit_start"] = rule_config["present"]
             elif rule == "document-end":
                 config["explicit_end"] = rule_config["present"]
             elif rule == "line-length":
                 config["width"] = rule_config["max"]
+            elif rule == "braces":
+                min_spaces_inside = rule_config["min-spaces-inside"]
+                if min_spaces_inside:
+                    config["min_spaces_inside"] = int(min_spaces_inside)
+                max_spaces_inside = rule_config["max-spaces-inside"]
+                if max_spaces_inside:
+                    config["max_spaces_inside"] = int(max_spaces_inside)
             elif rule == "indentation":
                 indent_sequences = rule_config["indent-sequences"]
                 # one of: bool, "whatever", "consistent"
@@ -669,6 +929,7 @@ class FormattedYAML(YAML):
     def dumps(self, data: Any) -> str:
         """Dump YAML document to string (including its preamble_comment)."""
         preamble_comment: Optional[str] = getattr(data, "preamble_comment", None)
+        self._prevent_wrapping_flow_style(data)
         with StringIO() as stream:
             if preamble_comment:
                 stream.write(preamble_comment)
@@ -676,6 +937,50 @@ class FormattedYAML(YAML):
             text = stream.getvalue()
         return self._post_process_yaml(text)
 
+    def _prevent_wrapping_flow_style(self, data: Any) -> None:
+        if not isinstance(data, (CommentedMap, CommentedSeq)):
+            return
+        for key, value, parent_path in nested_items_path(data):
+            if not isinstance(value, (CommentedMap, CommentedSeq)):
+                continue
+            fa: Format = value.fa  # pylint: disable=invalid-name
+            if fa.flow_style():
+                predicted_indent = self._predict_indent_length(parent_path, key)
+                predicted_width = len(str(value))
+                if predicted_indent + predicted_width > self.width:
+                    # this flow-style map will probably get line-wrapped,
+                    # so, switch it to block style to avoid the line wrap.
+                    fa.set_block_style()
+
+    def _predict_indent_length(
+        self, parent_path: List[Union[str, int]], key: Any
+    ) -> int:
+        indent = 0
+
+        # each parent_key type tells us what the indent is for the next level.
+        for parent_key in parent_path:
+            if isinstance(parent_key, int) and indent == 0:
+                # root level is a sequence
+                indent += self.sequence_dash_offset
+            elif isinstance(parent_key, int):
+                # next level is a sequence
+                indent += cast(int, self.sequence_indent)
+            elif isinstance(parent_key, str):
+                # next level is a map
+                indent += cast(int, self.map_indent)
+
+        if isinstance(key, int) and indent == 0:
+            # flow map is an item in a root-level sequence
+            indent += self.sequence_dash_offset
+        elif isinstance(key, int) and indent > 0:
+            # flow map is in a sequence
+            indent += cast(int, self.sequence_indent)
+        elif isinstance(key, str):
+            # flow map is in a map
+            indent += len(key + ": ")
+
+        return indent
+
     # ruamel.yaml only preserves empty (no whitespace) blank lines
     # (ie "/n/n" becomes "/n/n" but "/n  /n" becomes "/n").
     # So, we need to identify whitespace-only lines to drop spaces before reading.
