@@ -1,10 +1,14 @@
 """Test a set of 3rd party Ansible repositories for possible regressions."""
 import os
 import pathlib
+import re
+import shlex
 import subprocess
 
 import pytest
 
+from ansiblelint.testing import run_ansible_lint
+
 eco_repos = {
     "bootstrap": [
         "https://github.com/robertdebock/ansible-role-bootstrap",
@@ -45,12 +49,35 @@ def test_eco(repo: str) -> None:
         subprocess.run("git pull", cwd=f"{cache_dir}/{repo}", shell=True, check=True)
     else:
         subprocess.run(f"git clone {url} {cache_dir}/{repo}", shell=True, check=True)
-    # run ansible lint
-    subprocess.run(
-        f"ansible-lint 2>&1 > {my_dir}/{repo}.result",
-        shell=True,
-        check=False,
+    # run ansible lint and paths from user home in order to produce
+    # consistent results regardless on its location.
+
+    # we exclude `fqcn-builtins` until repository owners fix it.
+    args = ["-f", "pep8", "-x", "fqcn-builtins"]
+    result = run_ansible_lint(
+        *args,
+        executable="ansible-lint",
         cwd=f"{cache_dir}/{repo}",
     )
+
+    def sanitize_output(text: str) -> str:
+        """Make the output less likely to vary between runs or minor changes."""
+        # replace full path to home directory with ~.
+        result = text.replace(os.path.expanduser("~"), "~")
+        # removes summary line it can change too often on active repositories.
+        result = re.sub(r"^Finished with .+\n", "", result, flags=re.MULTILINE)
+
+        return result
+
+    result_txt = f"CMD: {shlex.join(result.args)}\n\nRC: {result.returncode}\n\nSTDERR:\n{result.stderr}\n\nSTDOUT:\n{result.stdout}"
+
+    with open(f"{my_dir}/{repo}.result", "w", encoding="utf-8") as f:
+        f.write(sanitize_output(result_txt))
     # fail if result is different than our expected one:
-    subprocess.run(f"git diff HEAD test/eco/{repo}.result", shell=True, check=True)
+    result = subprocess.run(
+        f"git diff --exit-code test/eco/{repo}.result",
+        shell=True,
+        check=False,
+        capture_output=True,
+    )
+    assert result.returncode == 0, result_txt
